@@ -57,7 +57,10 @@ struct _AnimeCursor{
             
 struct _GMozillaWebCertContext
 {
-    gpointer cert;
+    gpointer mozcert;
+    gpointer x509cert;
+    gpointer rawcert;
+    guint rawlen;
     guint error;
 };
 
@@ -76,24 +79,21 @@ struct _GMozillaWeb
     AnimeCursor *anime_cursor;
     GdkCursor *default_cursor;
     gboolean connected;
+    gpointer osso_context;
 };
 
 struct _GMozillaWebClass
 {
     GObjectClass parent;
-    void (*ask_cookie)                 (GObject *, gpointer, gint, const gchar *,
-                                        const gchar *, const gchar *, const gchar *,
-                                        const gchar *, gboolean, gint, GObject *);
     gint (*modal_dialog)               (GObject *, const gchar *, const gchar *,
                                         gint, gint, gint, gint);
     gboolean (*certificate_dialog)     (GObject *, GObject *);
     void (*certificate_password_dialog)(GObject *, const gchar *,
                                         const gchar *, gchar **);
-    void (*certificate_details)        (GObject *, gpointer);
-    gint (*on_submit)                  (GObject *);
-    gint (*select_match)               (GObject *, gpointer);
+    void (*certificate_details)        (GObject *, gchar *);
     void (*history_added)              (GObject *, const gchar *, GObject *);
     void (*connectivity_status)        (GObject *, gint);
+    gint (*on_service_notify)          (GObject *, gpointer ctx, const gchar *topic, const gchar *data);
 };
 
 GType       g_mozilla_web_get_type      (void);
