@@ -271,15 +271,6 @@ g_mozilla_transfer_item_class_init (GMozillaTransferItemClass *klass)
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE,
                       0);
-    transfer_item_signals[TRANSFER_ITEM_ERROR_SIGNAL] =
-        g_signal_new ("error",
-                      G_OBJECT_CLASS_TYPE (object_class),
-                      G_SIGNAL_RUN_FIRST,
-                      G_STRUCT_OFFSET (GMozillaTransferItemClass, error),
-                      NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
-                      G_TYPE_NONE,
-                      0);
     transfer_item_signals[TRANSFER_ITEM_ABORTED_SIGNAL] =
         g_signal_new ("aborted",
                       G_OBJECT_CLASS_TYPE (object_class),
@@ -487,7 +478,7 @@ gboolean progress_timeout (GMozillaTransferItem *self)
     reset_progress_timeout (self);
 
     // forcing download to get paused
-    g_signal_emit_by_name (G_OBJECT (self), "error");
+    g_signal_emit_by_name (G_OBJECT (self), "aborted");
     return FALSE;
 }
 
