@@ -40,6 +40,7 @@
 #include "gmozillapassman.h"
 #include <glib/gstdio.h>
 //#include "gmozillaconnectivity.c"
+#include "gtkmozembed_prompter.h" 
 
 #ifdef USE_OPENSSL
 #include <openssl/x509.h>
@@ -64,11 +65,19 @@ enum
     ICON_SIGNAL,
     MODAL_DIALOG,
     CONNECTIVITY_STATUS,
+    ON_SERVICE_NOTIFY,
     LAST_SIGNAL                   /** < Last signal used */
 };
 
 guint g_mozilla_web_signals[LAST_SIGNAL] = { 0 };
 
+typedef struct _GWebCookieContext GWebCookieContext;
+struct _GWebCookieContext
+{
+  gboolean remember_decision;
+  gboolean accept;
+};
+
 #ifdef MOZEAL_LOGGING
 static gboolean bmozeal_trace = FALSE;
 void
@@ -120,6 +129,11 @@ g_mozilla_web_destroy (GMozillaWeb *self)
 
     g_mozilla_connectivity_unset_global(self);
     gtk_moz_embed_pop_startup();
+#ifdef USE_HILDON
+    if (self->osso_context)
+        osso_deinitialize(self->osso_context);
+#endif
+    self->osso_context = NULL;
     return;
 }
 
@@ -538,11 +552,7 @@ g_mozilla_web_clear_cache (GMozillaWeb *self)
 {
     TRACE_LOG();
     //guint memoryCache = 1;
-    /*IMP
-      if (!gtk_moz_embed_common_clear_cache(GTK_MOZ_EMBED_COMMON(self->global), memoryCache))
-      return G_WEB_ERR;
-      */
-    if (!gtk_moz_embed_common_clear_cache())
+    if (gtk_moz_embed_common_clear_cache())
         return G_WEB_ERR;
 
     return G_WEB_STATUS_OK;
@@ -572,6 +582,7 @@ g_mozilla_web_set_new_window_loads (GMozillaWeb *self, gboolean loads)
 
     if (!gtk_moz_embed_common_set_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_USE_DEFAULT_HOME_PAGE, &loads))
         return G_WEB_ERR;
+    gtk_moz_embed_common_save_prefs();
 
     return G_WEB_STATUS_OK;
 }
@@ -627,10 +638,14 @@ static GCookieAcceptance
 g_mozilla_web_get_cookie_acceptance (GMozillaWeb *self)
 {
     TRACE_LOG();
-    gint cookie_behaviour = 0;
-    gint cookie_lifetime = 0;
-    gtk_moz_embed_common_get_pref(G_TYPE_INT, G_MOZILLA_PREF_COOKIE_BEHAVIOUR, &cookie_behaviour);
-    gtk_moz_embed_common_get_pref(G_TYPE_INT, G_MOZILLA_PREF_COOKIE_LIFETIMEPOLICY, &cookie_lifetime);
+    gint cookie_behaviour;
+    gint cookie_lifetime;
+
+    if ( !gtk_moz_embed_common_get_pref(G_TYPE_INT, G_MOZILLA_PREF_COOKIE_BEHAVIOUR, &cookie_behaviour) )
+       cookie_behaviour = 0;
+    if ( !gtk_moz_embed_common_get_pref(G_TYPE_INT, G_MOZILLA_PREF_COOKIE_LIFETIMEPOLICY, &cookie_lifetime) )
+       cookie_lifetime = 0;
+
     if (cookie_behaviour == 2)
     {
         return G_ACCEPT_COOKIE_NEVER;
@@ -644,10 +659,8 @@ g_mozilla_web_get_cookie_acceptance (GMozillaWeb *self)
         else
             return G_ACCEPT_COOKIE_PROMPT;
     }
-    else
-    {
-        return G_ACCEPT_COOKIE_PROMPT;
-    }
+
+    return G_ACCEPT_COOKIE_PROMPT;
 }
 
 #ifdef UNUSED
@@ -852,12 +865,14 @@ g_mozilla_web_confirm_cookie (GMozillaWeb *self, gpointer context, gboolean acce
     TRACE_LOG();
 
     if (action == G_WEB_COOKIE_REFUSE_ALL_FROM_SERVER || action == G_WEB_COOKIE_NO_ACTION) 
-        ((GtkMozEmbedCookie*) context)->accept = FALSE;
+        ((GWebCookieContext*) context)->accept = FALSE;
     else
-        ((GtkMozEmbedCookie*) context)->accept = TRUE;
+        ((GWebCookieContext*) context)->accept = TRUE;
 
     if (action == G_WEB_COOKIE_ACCEPT_ALL_FROM_SERVER || action == G_WEB_COOKIE_REFUSE_ALL_FROM_SERVER)
-        ((GtkMozEmbedCookie*) context)->remember_decision = TRUE;
+        ((GWebCookieContext*) context)->remember_decision = TRUE;
+    else
+        ((GWebCookieContext*) context)->remember_decision = FALSE;
 
     return G_WEB_STATUS_OK;
 }
@@ -916,7 +931,7 @@ g_mozilla_web_perform_crtificate_context_action (GMozillaWeb *self,
     TRACE_LOG();
     if (action == G_WEB_CERT_ACTION_ACCEPT) {
         GMozillaWebCertContext* ctx = (GMozillaWebCertContext*) context;
-        if (ctx->cert != cert) {
+        if (cert && ctx->mozcert != cert) {
             return G_WEB_CERT_REACTION_CLOSE;
         }
     } else if (action == G_WEB_CERT_ACTION_CANCEL) {
@@ -950,7 +965,7 @@ g_mozilla_web_get_certificate_iter (GMozillaWeb *self, GObject** iterator)
     GList * iter = (GList*)*iterator;
     if (iter->data) {
         GMozillaWebCertContext* context = (GMozillaWebCertContext*)iter->data;
-        return (GObject*)context->cert;
+        return (GObject*)context->mozcert;
     } else {
         return NULL;
     }
@@ -987,13 +1002,34 @@ g_mozilla_web_notify_plugins (GMozillaWeb *self, gint event)
        status = g_mozilla_plugin_enable_plugins (&self->disabled_pllist);
     }
     if(status)
-      gtk_moz_embed_common_reload_plugins ();
+      gtk_moz_embed_common_reload_plugins (FALSE);
     if (G_WEB_PLUGINS_LOW_MEMORY != event)
      return;
     gtk_moz_embed_common_observe (NULL, NULL, "memory-pressure", (gunichar *) "h\0e\0a\0p\0-\0m\0i\0n\0i\0m\0i\0z\0e\0\0");
     malloc_trim (0);
 }
 
+static void
+g_mozilla_web_notify_service (GMozillaWeb *self, const gchar *service_name, gpointer *context, const gchar *topic, gchar *data)
+{
+    ULOG_DEBUG_F("service_name:%s, topic:%s, ctx:%p", service_name, topic, context);
+
+    if(0 == g_ascii_strcasecmp(topic, "__focus_out__"))
+    {
+      gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+      return;
+    }
+
+    glong items_written = 0;
+    glong items_read = 0;
+    gunichar *name = NULL;
+    if (data)
+        name = g_utf8_to_utf16 (data, strlen (data), &items_read, &items_written, NULL);
+    gtk_moz_embed_common_observe (service_name, context, topic, name);
+    EAL_IF_GFREE(name);
+}
+
+
 static GWebStatus
 g_mozilla_web_set_user_agent_id (GMozillaWeb *self, gint ua_id)
 {
@@ -1053,13 +1089,13 @@ g_mozilla_web_username_matchinfo_delete (GMozillaWeb *self,  GObject* info, gint
         if (!index && (num > 1)) { // 0
 
             *((GList*) info) = *(g_list_nth ((GList*)info, index+1));
-            info = (GObject*)g_list_remove ((GList*) info, g_list_nth_data ((GList*) info, num));
+            g_list_remove ((GList*) info, g_list_nth_data ((GList*) info, num));
 
         } else if (num == 1)
 
             ((GList *) info)->data = NULL;
         else
-            info = (GObject*)g_list_remove ((GList*) info, g_list_nth_data ((GList*) info, index));
+            g_list_remove ((GList*) info, g_list_nth_data ((GList*) info, index));
     }
 
     // engine part !!!
@@ -1190,7 +1226,7 @@ g_mozilla_web_set_plugin_enabled (GMozillaWeb *self, gchar *name, gboolean enabl
 {
     TRACE_LOG();
     if (g_mozilla_plugin_set_item_status(name, enabled)) {
-        gtk_moz_embed_common_reload_plugins ();
+        gtk_moz_embed_common_reload_plugins (enabled);
         return G_WEB_STATUS_OK;
     }
     return G_WEB_ERR;
@@ -1393,6 +1429,8 @@ g_mozilla_web_interface_init (GWebIface *iface)
         g_mozilla_web_set_user_agent_id;
     iface->get_user_agent_id                  = (gint (*) (GWeb*))
         g_mozilla_web_get_user_agent_id;
+    iface->notify_service                     = (void (*) (GWeb*, const gchar *, gpointer *, const gchar *, gchar *))
+        g_mozilla_web_notify_service;
 }
 
 
@@ -1459,24 +1497,6 @@ g_mozilla_web_class_init (GMozillaWebClass * klass)
     object_class = (GtkObjectClass *) klass;
     widget_class = (GtkWidgetClass *) klass;
 
-    g_mozilla_web_signals[ASK_COOKIE] =
-        g_signal_new("ask-cookie",
-                     G_OBJECT_CLASS_TYPE (gobject_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(GMozillaWebClass, ask_cookie),
-                     NULL, NULL,
-                     g_mozeal_marshal_VOID__POINTER_INT_STRING_STRING_STRING_STRING_STRING_BOOLEAN_INT,
-                     G_TYPE_NONE, 9,
-                     G_TYPE_POINTER,
-                     G_TYPE_INT,
-                     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                     G_TYPE_BOOLEAN,
-                     G_TYPE_INT);
-
     g_mozilla_web_signals[CERTIFICATE_DIALOG] =
         g_signal_new("certificate-dialog",
                      G_OBJECT_CLASS_TYPE (gobject_class),
@@ -1504,8 +1524,8 @@ g_mozilla_web_class_init (GMozillaWebClass * klass)
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET(GMozillaWebClass, certificate_details),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__POINTER,
-                     G_TYPE_NONE, 1, G_TYPE_POINTER);
+                     g_cclosure_marshal_VOID__STRING,
+                     G_TYPE_NONE, 1, G_TYPE_STRING);
 
     g_mozilla_web_signals[HISTORY_ADDED] =
         g_signal_new("global-history-item-added",
@@ -1517,23 +1537,6 @@ g_mozilla_web_class_init (GMozillaWebClass * klass)
                      G_TYPE_NONE, 1,
                      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 
-    g_mozilla_web_signals[ON_SUBMIT_SIGNAL] =
-        g_signal_new("on-submit",
-                     G_OBJECT_CLASS_TYPE (gobject_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(GMozillaWebClass, on_submit),
-                     NULL, NULL,
-                     g_mozeal_marshal_INT__VOID,
-                     G_TYPE_INT, 0);
-    g_mozilla_web_signals[SELECT_MATCH_SIGNAL] =
-        g_signal_new("select-match",
-                     G_OBJECT_CLASS_TYPE (gobject_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(GMozillaWebClass, select_match),
-                     NULL, NULL,
-                     g_mozeal_marshal_INT__POINTER,
-                     G_TYPE_INT, 1, G_TYPE_POINTER);
-
     g_mozilla_web_signals[MODAL_DIALOG] =
         g_signal_new(G_WEBWIDGET_SIGNAL_MODAL_DIALOG,
                      G_OBJECT_CLASS_TYPE (gobject_class),
@@ -1554,6 +1557,16 @@ g_mozilla_web_class_init (GMozillaWebClass * klass)
                      g_cclosure_marshal_VOID__INT,
                      G_TYPE_NONE, 1, G_TYPE_INT);
 
+    g_mozilla_web_signals[ON_SERVICE_NOTIFY] =
+        g_signal_new("on-service-notify",
+                     G_OBJECT_CLASS_TYPE (gobject_class),
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(GMozillaWebClass, on_service_notify),
+                     NULL, NULL,
+                     g_mozeal_marshal_INT__POINTER_STRING_STRING,
+                     G_TYPE_INT, 3, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING);
+
+
 }
 
 static gboolean
@@ -1566,70 +1579,95 @@ gtk_moz_common_certificate_error_cb(GtkMozEmbedCommon *common, GObject * aCert,
     gint len = 0;
     unsigned char *p = gtk_moz_embed_common_nsx509_to_raw((gpointer)aCert, &len);
 
-    gpointer certPtr = NULL;
+    gpointer certPtr = d2i_X509 (NULL, (unsigned char **) &p, len);
 
-#ifndef NEW_OPENSSL
-    certPtr = d2i_X509 (NULL, (unsigned char **)&p, len);
-#else
-    certPtr = d2i_X509 (NULL, (unsigned char **) &p, len);
-#endif
-
-    g_signal_emit(G_OBJECT(self), g_mozilla_web_signals[CERTIFICATE_DETAILS], 0, certPtr);
     GMozillaWebCertContext *context = g_new0(GMozillaWebCertContext, 1);
-    context->cert = certPtr;
     context->error = aError;
+    context->mozcert = aCert;
+    context->x509cert = certPtr;
+    context->rawcert = p;
+    context->rawlen = len;
     self->mCertContexts = g_list_append(self->mCertContexts, context);
+    //g_signal_emit(G_OBJECT(self), g_mozilla_web_signals[CERTIFICATE_DETAILS], 0, p);
 
 #ifdef USE_CST
     // validating CERTIFICATE with maemo CertMan DB first.
     CST* storage;
+    gint result = CST_STATE_NOTVALID;
     cst_t_seqnum id;
-    storage = CST_open_file("/usr/share/certs/certman.cst", FALSE, NULL);
+    storage = CST_open(FALSE, NULL);
 
-    // Get stack representing the certificate chain
-    GSList* certificates = CST_get_chain(storage, certPtr);
-    if (certificates) 
-        return ret_val;
+    // check if the peer certificate is trusted (does not care if it's expired or revoked)
+    if (CST_is_valid(storage, certPtr))
+    {
+        result = CST_STATE_VALID;
+        // check if the certificate exists (it necessarily doesn't) and if it has sane trust settings
+        id = CST_search_by_X509(storage, certPtr);
+        if (id && 
+            !(CST_is_purpose(storage, id, CST_PURPOSE_SSL_SERVER) || 
+              CST_is_purpose(storage, id, CST_PURPOSE_SSL_CLIENT) ||
+              CST_is_purpose(storage, id, CST_PURPOSE_SSL_WLAN)))
+            result = CST_STATE_NOTVALID;
+    }
 
-    id = CST_search_by_X509(storage, certPtr);
-    // Check if certificate exists and if it is meant for browser or wlam connections.
-    if (!id ||
-        (!(CST_is_purpose(storage, id, CST_PURPOSE_SSL_SERVER) || CST_is_purpose (storage, id, CST_PURPOSE_SSL_CLIENT)) &&
-        (!(CST_is_purpose(storage, id, CST_PURPOSE_SSL_WLAN)))))
-#endif
+    CST_free(storage);
+
+    if (result != CST_STATE_VALID)
+#endif /* USE_CST */
          g_signal_emit(G_OBJECT(self), g_mozilla_web_signals[CERTIFICATE_DIALOG], 0, context, &ret_val);
 
     return ret_val;
 };
 
-static gint gtk_moz_common_username_select_login_cb(GtkMozEmbedCommon *common, GList * logins, GMozillaWeb *self)
+static gint gtk_moz_common_username_select_login_cb(GtkMozEmbedCommon *common, GtkMozEmbed *mozembed, GList * logins, GMozillaWeb *selfweb)
 {
     TRACE_LOG();
     gint ret_val = -1;
-    g_signal_emit(G_OBJECT(self), g_mozilla_web_signals[SELECT_MATCH_SIGNAL], 0, (const gpointer)logins, &ret_val);
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
+    g_signal_emit_by_name(G_OBJECT(self), "select-match", (const gpointer)logins, &ret_val);
     return ret_val;
 };
 
-static gint gtk_moz_common_remember_login_cb (GtkMozEmbedCommon *common, GMozillaWeb *self)
+static gint gtk_moz_common_remember_login_cb (GtkMozEmbedCommon *common, GtkMozEmbed *mozembed, GMozillaWeb *selfweb)
 {
     TRACE_LOG();
     gint ret_val = -1;
-    g_signal_emit (G_OBJECT(self), g_mozilla_web_signals[ON_SUBMIT_SIGNAL], 0, &ret_val);
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
+    g_signal_emit_by_name(G_OBJECT(self), "on-submit", &ret_val);
 
     return ret_val;
 };
 
-static void gtk_moz_common_ask_cookie_cb (GtkMozEmbedCommon *common, gpointer context, 
-                                     gint actions, const gchar * url,
-                                     const gchar * name, const gchar * value,
-                                     const gchar * domain, const gchar * path,
-                                     gboolean illegal_path, gint expire, 
-                                     GMozillaWeb *self) {
+static void gtk_moz_common_ask_cookie_cb (GtkMozEmbedCommon *common, GtkMozEmbed *mozembed,
+                                          gboolean *remember,
+                                          gint *action,
+                                          gint expire, 
+                                          const gchar * url,
+                                          const gchar * name, const gchar * value,
+                                          const gchar * domain, const gchar * path,
+                                          GMozillaWeb *selfweb)
+{
 
     TRACE_LOG();
-    g_signal_emit_by_name(GTK_OBJECT(self), "ask-cookie", context, actions, url, name, value, domain, path, illegal_path, expire);
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
 
-    return ;
+    GWebCookieContext * ctx = g_new0(GWebCookieContext*, 1);
+    ctx->remember_decision = remember?*remember:FALSE;
+    ctx->accept = action?*action:0;
+
+    g_signal_emit_by_name(G_OBJECT(self), "ask-cookie", ctx, 1, url, name, value, domain, path, FALSE, expire);
+    
+    if (action)
+        *action = !ctx->accept;
+    if (remember)
+        *remember = ctx->remember_decision;
+    
+    g_free(ctx);
+
+    return;
 }
 
 /*
@@ -1658,6 +1696,169 @@ static void gtk_moz_common_ask_cookie_cb (GtkMozEmbedCommon *common, gpointer co
    };
    */
 
+static gboolean
+prompt_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dialog_title,
+           const gchar * dialog_msg, gchar ** default_msg,
+           const gchar * check_msg, gboolean * check_value,
+           GMozillaWeb *selfweb)
+{
+
+    TRACE_LOG ();
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
+
+    gchar *response = NULL;
+    g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_PROMPT,
+                           dialog_msg, *default_msg, &response, NULL);
+
+    if (*default_msg && response)
+    {
+        EAL_GFREE (*default_msg);
+        *default_msg = g_strdup_printf ("%s", response);
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static gint
+confirm_ex_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed,
+               const char *title, const char *text, guint bt_flags,
+               const char *button1, const char *button2, const char *button3,
+               const char *check_msg, gboolean * check_val,
+               GMozillaWeb *selfweb)
+{
+
+    TRACE_LOG ();
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), 1);
+
+    gboolean ret_val;
+    gchar *ntext = (gchar *) text;
+    gint cur = 0, i = 0;
+    gboolean prev_space = FALSE, isspace = FALSE;
+    for (i = 0; i != strlen (text); i++)
+    {
+        isspace = g_ascii_isspace (text[i]);
+        if (!isspace || !prev_space)
+            ntext[cur++] = text[i];
+        prev_space = isspace;
+    }
+    ntext[cur] = 0;
+    g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_CONFIRM, ntext,
+                           &ret_val, NULL);
+    return ret_val ? 0 : 1;
+}
+
+static gboolean
+prompt_auth_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dialog_title,
+                const gchar * dialog_msg, gchar ** user_name,
+                gchar ** user_passwd, const gchar * check_msg,
+                gboolean * check_value, GMozillaWeb *selfweb)
+{
+    TRACE_LOG ();
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
+
+    GList *logins = NULL;
+
+    EAL_IF_GFREE (self->username);
+    EAL_IF_GFREE (self->password);
+    self->accept = FALSE;
+
+    gint logins_count = gtk_moz_embed_common_get_logins (dialog_title, &logins);
+
+    selfweb->logins_list = NULL;
+    if (logins_count)
+        selfweb->logins_list = logins;
+
+    // FIXME: please, that is an workaround for bug 42663, so remove it as soon as it gets solved in a another way.                                                                                                                                                                                                                                                                                                                                                                                                             
+    gint authentification_type;
+
+    gchar *temp_current_location = gtk_moz_embed_get_location (mozembed);
+
+    if (self->last_location
+        && !strcmp (self->last_location, temp_current_location))
+        authentification_type = G_WEBENGINE_AUTHENTICATION_WRONG;
+    else
+        authentification_type = 0;
+
+    EAL_IF_GFREE (temp_current_location);
+
+    g_signal_emit_by_name (G_OBJECT (self),
+                           G_WEBWIDGET_SIGNAL_AUTHENTIFICATION_REQUIRED,
+                           authentification_type, dialog_title, dialog_msg);
+
+    GList *ptr = g_list_first (logins);
+    while (ptr)
+    {
+        GtkMozLogin *login = (GtkMozLogin *) ptr->data;
+        EAL_IF_GFREE (login->user);
+        EAL_IF_GFREE (login->pass);
+        EAL_IF_GFREE (login->host);
+        ptr = ptr->next;
+    }
+
+    if (self->accept)
+    {
+        if (self->username && *self->username)
+        {
+            if (user_name)
+            {
+                if (*user_name)
+                    g_free (*user_name);
+                *user_name = g_strdup (self->username);
+                EAL_GFREE (self->username);
+            }
+        }
+        if (self->password && *self->password)
+        {
+            if (user_passwd)
+            {
+                if (*user_passwd)
+                    g_free (*user_passwd);
+                *user_passwd = g_strdup (self->password);
+                EAL_GFREE (self->password);
+            }
+        }
+        self->accept = FALSE;
+        gboolean rremember;
+        if (!gtk_moz_embed_common_get_pref
+            (G_TYPE_BOOLEAN, G_MOZILLA_PREF_REMEMBER_PASSWORDS, &rremember))
+            rremember = FALSE;
+        *check_value = rremember;
+        return TRUE;
+    }
+    return FALSE;
+    // workaround for bug 46012                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
+}
+
+static gboolean
+select_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dialog_title,
+           const gchar * dialog_text, gpointer dialog_options,
+           gpointer dialog_choice, GMozillaWeb *selfweb)
+{
+    TRACE_LOG ();
+    GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
+    g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
+
+    gboolean ret_val;
+    g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_SELECT,
+                           dialog_title, dialog_text, dialog_options,
+                           dialog_choice, &ret_val, NULL);
+    return ret_val;
+}
+
+static gint
+on_service_notify_cb (GtkMozEmbedCommon * embed, gpointer ctx, const gchar * topic,
+		      const gchar * data, GMozillaWeb * self)
+{
+    TRACE_LOG();
+    gint retval = 0;
+    g_signal_emit (G_OBJECT (self), g_mozilla_web_signals[ON_SERVICE_NOTIFY],
+	               0, ctx, topic, data, &retval);
+    return retval;
+}
+
 static void
 g_mozilla_web_global_coonect(GMozillaWeb *self)
 {
@@ -1668,8 +1869,17 @@ g_mozilla_web_global_coonect(GMozillaWeb *self)
                       G_CALLBACK (gtk_moz_common_username_select_login_cb), self);
     g_signal_connect (G_OBJECT (self->global), "remember-login",
                       G_CALLBACK (gtk_moz_common_remember_login_cb), self);
-    g_signal_connect (G_OBJECT (self->global), "ask-cookie",
-                      G_CALLBACK (gtk_moz_common_ask_cookie_cb), self);
+    g_signal_connect (G_OBJECT (self->global), "on_service_notify", G_CALLBACK (on_service_notify_cb), self);
+
+    GtkMozPrompter *obj = gtk_moz_prompter_get ();
+    g_signal_connect (G_OBJECT (obj), "cookie-dialog", G_CALLBACK (gtk_moz_common_ask_cookie_cb), self);
+    g_signal_connect (G_OBJECT (obj), "alert", G_CALLBACK (alert_cb), self);
+    g_signal_connect (G_OBJECT (obj), "confirm", G_CALLBACK (confirm_cb), self);
+    g_signal_connect (G_OBJECT (obj), "confirm_ex", G_CALLBACK (confirm_ex_cb), self);
+    g_signal_connect (G_OBJECT (obj), "prompt", G_CALLBACK (prompt_cb), self);
+    g_signal_connect (G_OBJECT (obj), "prompt_auth", G_CALLBACK (prompt_auth_cb), self);
+    g_signal_connect (G_OBJECT (obj), "select", G_CALLBACK (select_cb), self);
+
 
     //    g_signal_connect (G_OBJECT (self->global), "modal-dialog",
     //                      G_CALLBACK (gtk_moz_common_modal_dialog_cb), self);
@@ -1725,8 +1935,14 @@ g_mozilla_web_set_default_config (GMozillaWeb *self)
     if (!gtk_moz_embed_common_get_pref(G_TYPE_BOOLEAN, "snav.disableJS", &bval))
         gtk_moz_embed_common_set_pref(G_TYPE_BOOLEAN, "snav.disableJS", &bfval);
 
+    if (!gtk_moz_embed_common_get_pref(G_TYPE_STRING, "snav.block", &sval))
+        gtk_moz_embed_common_set_pref(G_TYPE_STRING, "snav.block", "http://www.google.com/reader http://docs.google.com");
+    else g_free(sval);
+    sval = NULL;
+
     gval = 7;
-    gtk_moz_embed_common_set_pref (G_TYPE_INT, "accessibility.tabfocus", &gval);
+    if (!gtk_moz_embed_common_get_pref(G_TYPE_INT, "accessibility.tabfocus", &gval))
+        gtk_moz_embed_common_set_pref (G_TYPE_INT, "accessibility.tabfocus", &gval);
 
     // focus border styling
 /*    gval = 3;
@@ -1786,6 +2002,9 @@ g_mozilla_web_set_default_config (GMozillaWeb *self)
     if (!gtk_moz_embed_common_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_ENABLE_JAVA_SCRIPT, &bval))
         gtk_moz_embed_common_set_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_ENABLE_JAVA_SCRIPT, &btval);
 
+//    if (!gtk_moz_embed_common_get_pref(G_TYPE_BOOLEAN, "plugin.expose_full_path", &bval))
+//    gtk_moz_embed_common_set_pref(G_TYPE_BOOLEAN, "plugin.expose_full_path", &btval);
+
     if (!gtk_moz_embed_common_get_pref(G_TYPE_BOOLEAN, "layout.full.zoom.mode.fast", &bval))
         gtk_moz_embed_common_set_pref(G_TYPE_BOOLEAN, "layout.full.zoom.mode.fast", &bfval);
 
@@ -1885,14 +2104,18 @@ GObject*
 g_web_new (void)
 {
     TRACE_LOG();
-    GObject* instance = G_OBJECT(g_object_new(g_mozilla_web_get_type(), NULL));
+    GObject* instance = NULL;
+    instance = G_OBJECT(g_object_new(g_mozilla_web_get_type(), NULL));
     G_MOZILLA_WEB(instance)->global = G_OBJECT(gtk_moz_embed_common_new());
     G_MOZILLA_WEB(instance)->is_configured = FALSE;
     g_mozilla_web_global_coonect(G_MOZILLA_WEB(instance));
+#ifdef USE_HILDON
+    G_MOZILLA_WEB(instance)->osso_context = osso_initialize ("osso_browserd", VERSION, TRUE, NULL);
+#endif
     return instance;
 }
 
-gboolean
+static gboolean
 g_mozilla_create_extensions_folder(void)
 {
     const gchar *name = NULL;
@@ -1926,6 +2149,27 @@ g_mozilla_create_extensions_folder(void)
     return TRUE;
 }
 
+static gboolean
+g_mozilla_check_system_extensions(const gchar *full_path, const gchar *profile_path)
+{
+    struct stat sb, sb2;
+    gboolean tt = FALSE;
+    if (stat("/var/lock/browser_extensions.stamp", &sb) != -1) {
+        gchar * compreg = g_strdup_printf("%s/%s/%s", full_path, profile_path?profile_path:"microb", "compreg.dat");
+        if (stat(compreg, &sb2) != -1) {
+          if (sb.st_mtime > sb2.st_mtime) {
+              gchar * xpti = g_strdup_printf("%s/%s/%s", full_path, profile_path?profile_path:"microb", "xpti.dat");
+              unlink(compreg);
+              unlink(xpti);
+              g_free(xpti);
+              tt = TRUE;
+          }
+        }
+        g_free(compreg);
+    }
+    return tt;
+}
+
 void
 g_web_set_env (void)
 {
@@ -1933,6 +2177,8 @@ g_web_set_env (void)
     gtk_moz_embed_set_path(PREFIX"/lib/microb-engine");
     char *home_path;
     char *full_path;
+    char *comp_path = NULL;
+    char *profile_path = NULL;
 
     home_path = (char*)getenv("HOME");
     if (!home_path)
@@ -1940,17 +2186,22 @@ g_web_set_env (void)
         fprintf(stderr, "Failed to get HOME\n");
         exit(1);
     }
-    gchar *plugins_path = g_strdup_printf("%s/%s/%s", getenv ("HOME"), ".mozilla", "plugins");
+    comp_path = (char*)getenv("MOZ_COMP_PATH_OVERRIDE");
+    profile_path = (char*)getenv("MOZ_PROFILE_OVERRIDE");
+
+    gchar *plugins_path = g_strdup_printf("%s/%s/%s", getenv ("HOME"), comp_path?comp_path:".mozilla", "plugins");
     setenv("MOZILLA_INTERNAL_PLUGINS_DIR", plugins_path, 0);
-    gchar *extensions_path = g_strdup_printf("%s/%s/%s", getenv ("HOME"), ".mozilla/microb", "extensions");
+    gchar *extensions_path = g_strdup_printf("%s/%s/%s/%s", getenv ("HOME"), comp_path?comp_path:".mozilla", profile_path?profile_path:"microb", "extensions");
     setenv("MOZILLA_INTERNAL_EXTENSIONS_DIR", extensions_path, 0);
     setenv("MOZ_PLUGIN_PATH", plugins_path, 0);
     setenv("MOZ_DISABLE_SIG_HANDLER", "1", 0);
-    full_path = g_strdup_printf("%s/%s", home_path, ".mozilla");
-    gtk_moz_embed_set_profile_path(full_path, "microb");
+    full_path = g_strdup_printf("%s/%s", home_path, comp_path?comp_path:".mozilla");
+    gtk_moz_embed_set_profile_path(full_path, profile_path?profile_path:"microb");
     gtk_moz_embed_set_comp_path(full_path);
     symlink(OSSO_PLUGINS, plugins_path);
     g_mozilla_create_extensions_folder();
+    gboolean tt = g_mozilla_check_system_extensions(full_path, profile_path);
+
 #ifdef USE_HILDON
     /* Hack workaround (Hildon Gtk Theme s...) for setup proper bg[selected] style */
     gtk_rc_parse_string("style \"MicrobBG\" = \"osso-logical-colors\"\n"
@@ -1961,6 +2212,9 @@ g_web_set_env (void)
     g_free(extensions_path);
     g_free(full_path);
 	gtk_moz_embed_push_startup();
+//    if (tt)
+//      gtk_moz_embed_common_observe(NULL, "profile-before-change", NULL);
+    gtk_moz_common_microb_init();
 
     return;
 }
